
% Script calculates a forward encoding model on time domain EEG DATA
% (see Lalor 2006, 2009);

% Sophie Herbst, Feb. 2016
% ksherbst@gmail.com

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
clear all; close all; clc;

%% Directories
% load the path where fieldtrip is located
addpath('/Users/sophie/ownCloud/MATLAB_Toolboxes/fieldtrip-20160620/'); % using the newer FT version!
addpath('/Users/sophie/ownCloud/MATLAB_Toolboxes/shadedErrorBar');
ft_defaults

% load TRF functions
addpath('/Users/sophie/ownCloud/MATLAB_Toolboxes/mTRF_1.3')

rootDir = '/Volumes/SophiesDATA/DisTime_EEGII/';
rawDir = '/Volumes/SophiesDATA/DisTime_EEGII/raw/';
preprocDir = '/Volumes/SophiesDATA/DisTime_EEGII/preproc/';

PicDir = '/Users/sophie/ownCloud/PROJECT_DisTime/DisTime_mTRF/MS_DisTime_TRF/Pics_MS_DisTime_TRF/MatlabOutput';

%% Model Parameters

% run mTFR and prediction or just load the saved data
run_model = 1;

check_plots = 1; % make single subject plots

% lags and lambda for the TFR;
tau = [-200  600];
% timevector for the lags (used for plotting)
tlags = tau(1)/1000:1/200:tau(2)/1000;
nlags = size(tlags,2);

lambda = 4; %0.01;

% epoch time-locked to cross/cue or tone
lock = 'cross';

% latency for ft_selectdata
latency =[0 3.5];

% use only trials that have minimally this foreperiod duration
minFP = 0.65;

% take DATA only from that timepoint (postcue); avoid mixing in cue-resonse
startLAT = .5;

% subjects
sbs = dir([rawDir '*.cnt']);
%sbs(1:2) = [];

% define one channel for plotting
channame = 'Cz';

%% Audio Parameters

P.audiofreq = 44100;%48000;
P.noisefreq = P.audiofreq;
P.sounddur = 0.05;

%% initialize MATRICES sbsx conditions

B_MAT= cell(size(sbs,1),3); % Matrix for the regression weights
const_MAT = cell(size(sbs,1),3); % Matrix for the regression constant
origEEG = cell(size(sbs,1),3); % Matrix for original data
STIM_MAT = cell(size(sbs,1),3); % Matrix for stimuli
predEEG = cell(size(sbs,1),3); % Matrix for predicted EEG data

%% start loading DATA

if run_model
    for i_sb = 1 : size(sbs,1)
        
        % load raw DATA for each SB
        [dum SB] =  fileparts(sbs(i_sb).name);
        disp(['loading subject ' SB '...']);
        load([preprocDir filesep SB 'preproc_complete_' lock 'lock_HP_minphase']);
        
        DATA = DATA_clean; clear DATA_clean;
        
        plotchan = find(strcmp(DATA.label,channame));
        
        %% LP filter the data
        % try to use same filter for stimulus envelope  DATA
        
        cfg = [];
        cfg.lpfilter = 'yes';
        cfg.lpfilttype = 'but'; %'firws';
        
        cfg.lpfilord = 3;
        
        cfg.plotfiltresp = 'no';
        cfg.lpfiltdir = 'twopass'; %'onepass-zerophase';
        % cfg.lpfiltwintype = 'kaiser';
        
        cfg.lpfreq   = 25; %8% % low pass frequency
        cfg.lpfiltdf = 1; % transition band witdth
        
        DATA_filt = ft_preprocessing(cfg,DATA);
        
        if check_plots
            % check effect of filtering
            
            tmptime = DATA.time{1};
            
            cfg = [];
            DATA_avg = ft_timelockanalysis(cfg,DATA);
            DATA_filt_avg = ft_timelockanalysis(cfg,DATA_filt);
            
            tmpdat = DATA_avg.avg;
            tmpdat_filt = DATA_filt_avg.avg;
            
            figure();
            plot(tmptime,tmpdat(plotchan,:)); hold all;
            plot(tmptime,tmpdat_filt(plotchan,:));
            
            legend({'unfiltered', 'filtered'});
            title (['SB ' SB ' channel channame' ])
            
            clear DATA_avg DATA_filt_avg tmpdat tmpdat_filt
        end
        
        DATA = DATA_filt; clear DATA_filt;
        fs = round(DATA.fsample); % round because in some sbs fs = 200.00 and that causes prblms with mTRF
        
        
        % select DATA at the defined latencies
        cfg = [];
        cfg.latency = latency;
        DATA = ft_selectdata(cfg,DATA);
        Tvec = DATA.time{1}; % the data time vector
        
        % for TRF only data from 500ms after cue onset will be used to remove visual onset ERP
        startIX = nearest(Tvec,startLAT); % same for all trials
        endIX = size(Tvec,2); % find the last sampling point
        
        %% loop over conditions
        for i_cond = 1:3
            disp(['cond ' num2str(i_cond)]);
            
            %% prepare condition DATA
            clear condDATA;
            
            % select all trials from the condition and with FP >= minFP
            cfg = [];
            cfg.trials = find(DATA.trialinfo(:,1) == i_cond & DATA.trialinfo(:,3) >= minFP);
            condDATA = ft_selectdata(cfg,DATA);
            
            % initialize subjects's matrix for regression weights: trials x channels x lags
            B_MAT{i_sb,i_cond} = nan(size(condDATA.trial,2),61,nlags);
            const_MAT{i_sb,i_cond} = nan(size(condDATA.trial,2),61);
            
            STIM_MAT{i_sb,i_cond} = zeros(size(condDATA.trial,2),size(condDATA.trial{1},2)); % trials x TP
            origEEG{i_sb,i_cond}  = zeros(size(condDATA.trial,2),size(condDATA.trial{1},1),size(condDATA.trial{1},2)); % trials x chan x TP
            predEEG{i_sb,i_cond} = zeros(size(condDATA.trial,2),size(condDATA.trial{1},1),size(condDATA.trial{1},2)); % trials x chan x TP
            
            
            %% loop over trials
            for i_trial = 1:size(condDATA.trial,2)
                
                disp(['trial ' num2str(i_trial)]);
                
                trial_Tvec = Tvec(startIX:endIX); % time vector for the trial DATA
                trialFP = condDATA.trialinfo(i_trial,3);% FP on each trial
                
                %% take the data from this trial
                trialdat = condDATA.trial{i_trial}(:,startIX:endIX); % and cut out the relevant time interval
                
                %% prepare stimulus envelope
                
                % create sin tone at given frequency
                ToneFreq = condDATA.trialinfo(i_trial,4);
                sounds = ToneFreq;
                sound1 = [];
                
                sound1 = MakeBeep(sounds, P.sounddur, P.audiofreq);
                sound1 = fade_me(sound1,P.audiofreq,10,10); % let tones fade in
                
                % time vector of the tone
                timevec_tone = 0:1/P.audiofreq:P.sounddur;
                
                % make a trialtime vector at the same sampling rate as the tone
                trial_time_vec_audFS =  DATA.time{i_trial}(startIX):1/P.audiofreq:DATA.time{i_trial}(endIX);
                
                % create trial long vector of zeros and insert tone
                stim_vec_audFS = zeros(size(trial_time_vec_audFS));
                FPix = nearest(trial_time_vec_audFS,trialFP);
                stim_vec_audFS(FPix:FPix+size(sound1,2)-1) = sound1;
                
                % take the envelope
                stim_vec_audFS_H = abs(hilbert(stim_vec_audFS));
                
                % downsample to EEG frequency (by hand; using the matlab
                % downsample function gives different amount of timepoints)
                for x = 1:size(trial_Tvec,2)
                    % find closest timepoint-index for each timepoint of trial_Tvec
                    ds_IX(x) = nearest(trial_time_vec_audFS,trial_Tvec(x));
                end
                
                % take stim_vec at the defined indeces
                stim_vec = stim_vec_audFS_H(ds_IX);
                trial_time_vec = trial_time_vec_audFS(ds_IX); % this vector is the same as trial_Tvec with minimal differences (rounding)
                
                
                % filter the envelope
                [b,a] = butter(3,[25/(fs/2)],'low');
                %tic
                stim_vec_lp = filtfilt(b,a,stim_vec);
                %toc
                
                % 1. derivative
                stim_vec_dx = diff(stim_vec_lp);
                stim_vec_dx = [0 stim_vec_dx]; % first value dissapears
                
                % half wave rectify
                stim_vec_dx_hw = stim_vec_dx .*(stim_vec_dx >0) ;
                
                if check_plots
                    if i_trial == 1
                        %% plot stimulus envelope
                        
                        my_colors = parula(5);
                        
                        figure('Units', 'pixels', ...
                            'Position', [100 100 500 375]);
                        hold on;
                        
                        plot(trial_time_vec_audFS , stim_vec_audFS,'color',[0.1 0.1 0.1],'Linewidth',.01); hold all;
                        
                        % add lp filtered envelope
                        plot(trial_time_vec, stim_vec,'color',my_colors(2,:),'Linewidth',2); hold on;
                        
                        % add 1. der. envelope
                        plot(trial_time_vec, stim_vec_dx,'color',my_colors(3,:),'Linewidth',2); hold on;
                        
                        % add 1. der. envelope
                        plot(trial_time_vec, stim_vec_dx_hw,'color',my_colors(4,:),'Linewidth',2); hold on;
                        
                        legend('raw tone', 'lp filtered envelope', '1.derivative of envelope', 'halvw. rect 1.derivative','Location','NorthWest')
                        ylim([-1.2 1.2]);
                        set(gca,'ytick',[]);
                        set(gca,'yticklabels',[]);
                        
                        xlim([.65  trialFP + .2]);
                        set(gca,'xtick',[.7:.1:trialFP + .2]);
                        xlabel('time[s]');
                        title(['FP: ' num2str(trialFP)]);
                        
                        % aesthetics
                        set( gca                       , ...
                            'FontName'   , 'Helvetica' );
                        
                        set(gca, ...
                            'Box'         , 'off'     , ...
                            'TickDir'     , 'out'     , ...
                            'TickLength'  , [.02 .02] , ...
                            'XMinorTick'  , 'off'      , ...
                            'YMinorTick'  , 'off'      , ...
                            'YGrid'       , 'on'      , ...
                            'XColor'      , [.3 .3 .3], ...
                            'YColor'      , [.3 .3 .3], ...
                            'YTick'       , 0:500:2500, ...
                            'LineWidth'   , 1         );
                        
                        print('-dpdf',[PicDir filesep 'TRF_examplestim.pdf'])% PicDir filesep
                        
                        
                    end
                end
                
                %% zscore both timeseries, make time 1. dimension
                % zstim = zscore(stim_vec_dx_hw');
                zstim = (stim_vec_dx_hw ./ max(stim_vec_dx_hw))';
                ztrialdat = zscore(trialdat');
                
                %                 figure;
                %                 plot(trial_Tvec, zstim); hold all;
                %                 plot(trial_Tvec, ztrialdat(:,plotchan));
                %                 legend({'stimulus', 'EEG data'});
                %                 hline(0,'k');
                %                 title(['Example Stimulus and EEG DATA, 1 Trial']);
                % % %
                % write DATA and stim (HF) in big matrices
                STIM_MAT{i_sb,i_cond}(i_trial,startIX:endIX) = zstim';
                origEEG{i_sb,i_cond}(i_trial,:,startIX:endIX)  = ztrialdat';
                
                
                %% this is the TRF
                [X,T,const] =  mTRFtrain(zstim,ztrialdat,fs,0,tau(1), tau(2),lambda);
                
                % plot(T,squeeze(X(:,:,27)); % plot single trial mTRF
                
                % write the betas into a matrix
                B_MAT{i_sb,i_cond}(i_trial,:,:) = squeeze(X)';
                const_MAT{i_sb,i_cond}(i_trial,:) = const;
                
            end
            
            avgB_MAT(i_sb,i_cond,:,:) = squeeze(mean(B_MAT{i_sb,i_cond},1));
            
        end
        
        % if check_plots
        figure();
        for i_cond = 1:3
            TRF4plot = squeeze(avgB_MAT(i_sb,i_cond,plotchan,:));
            TRF4plot_sd = squeeze(std(B_MAT{i_sb,i_cond},1));
            
            
            plot(tlags,TRF4plot);hold all;
        end
        legend({'cond 1','cond 2','cond 3'});
        hline(0, 'k');
        title(['TRF SB ' SB ' lambda: ' num2str(lambda)]);
        xlabel('TRF lag [s]')
        % end
        
    end
    % save([dataDir filesep 'RES_EEGII_TRF_predTone_lambda' num2str(lambda)],'avgB_MAT')  ;
else
    load ([dataDir filesep 'RES_EEGII_TRF_predTone'])
end

%%
colors  = [246 143 39; 215 32 39; 44 130 186 ];%colormap(parula(3));
colors = colors./255;

%% plot avg TRFs
% GFP over lags

% compute GFP as std over channels
GFP = squeeze(std(avgB_MAT,[],3)); % dim 3 = channels

% avg over sbs and calculate SEM
avgGFP = squeeze(mean(GFP,1));
SEGFP = squeeze(std(GFP,[],1)/sqrt(22));

lags = tlags; % lag vector for plotting

% plot each cond
figure;
for i_cond = 1:3
    plot(lags,squeeze(avgGFP(i_cond,:)),'color',colors(i_cond,:)); hold all;
end
legend({'cond 1', 'cond 2', 'cond 3'})

for i_cond = 1:3
    shadedErrorBar(lags,squeeze(avgGFP(i_cond,:)),squeeze(SEGFP(i_cond,:)),{'color',colors(i_cond,:)},.5);
end

%ylim([0 ,.01])
xlabel('TRF lag [s]');

title(['TRF (lambda = ' num2str(lambda) ') : GFP']);

%% plot avg TRFs at one channel
cond_colors = [253,184,99; 178,171,210; 94,60,153];
%[230,97,1; 253,184,99; 178,171,210; 94,60,153];
cond_colors = cond_colors./255;

thischan = 'Cz';
plotchan = find(strcmp(DATA.label,thischan));

% compute GFP as std over channels
chan_TRF = squeeze(avgB_MAT(:,:,plotchan,:)); % dim 3 = channels

% avg over sbs and calculate SEM
avgTRF = squeeze(mean(chan_TRF,1));
seTRF = squeeze(std(chan_TRF,[],1)/sqrt(22));

lags = tlags; % lag vector for plotting

% plot each cond
figure('Units', 'pixels', ...
                            'Position', [100 100 500 375]);
                        hold on;
                        
for i_cond = 1:3
    plot(lags,squeeze(avgTRF(i_cond,:)),'color',cond_colors(i_cond,:),'LineWidth',1.5); hold all;
end
legend({'NP', 'p', 'P'},'Location','NorthWest');
legend('boxoff')


for i_cond = 1:3
    shadedErrorBar(lags,squeeze(avgTRF(i_cond,:)),squeeze(seTRF(i_cond,:)),{'color',cond_colors(i_cond,:)},.5);
end

ylim([-.15  .15])
xlim([-.1  .5])
xlabel('TRF lag [s]');
hl = hline(0,'k')
set(hl,'color',[.3 .3 .3])
vl = vline(0,'k')
set(vl,'color',[.3 .3 .3])


title(['TRF (lambda = ' num2str(lambda) ') at ' thischan]);

% aesthetics
                        set( gca                       , ...
                            'FontName'   , 'Helvetica' ,...
                            'FontSize'   , 10 );
                        
                        set(gca, ...
                            'Box'         , 'off'     , ...
                            'TickDir'     , 'out'     , ...
                            'TickLength'  , [.02 .02] , ...
                            'XMinorTick'  , 'on'      , ...
                            'YMinorTick'  , 'on'      , ...
                            'YGrid'       , 'on'      , ...
                            'XGrid'       , 'on'      , ...
                            'XColor'      , [.3 .3 .3], ...
                            'YColor'      , [.3 .3 .3], ...
                            'YTick'       , -.1: .1 : .1, ...
                            'LineWidth'   , .75         );
                        
print('-dpdf',[PicDir filesep 'EEGII_TRF_TONE' thischan   '.pdf'], '-painters')% PicDir filesep

%% plot topos of regression weights

% avg weights over sbs
allsbs_BMAT = squeeze(mean(avgB_MAT,1)); % first dim is now cond

% lags at which to plot
plottimes = [0.18 0.20];%[0 0]
plotlag1 =  nearest(lags,plottimes(1));
plotlag2 =  nearest(lags,plottimes(2));


figure;
for i_cond = 1:3
    
    % insert the topodata from each condition into a fieldtrip structure
    clear tmpdat TOPO
    tmpdat = DATA;
    
    cfg = [];
    cfg.latency = [1];
    cfg.trials = 1;
    tmpdat = ft_selectdata(cfg,tmpdat);
    
    cfg =[];
    TOPO = ft_timelockanalysis(cfg,tmpdat); % timelock
    
    TOPO.avg = squeeze(mean(allsbs_BMAT(i_cond,:,plotlag1:plotlag2),3))'; % replace by real data avg
    
    % ft_timelock that structure in the desired interval
    cfg = [];
    cfg.layout =  'elec1005.lay';
    cfg.colorbar = 'yes';
    cfg.parameter =  'avg';
    cfg.comment = 'no';
    cfg.zlim = 'absmax';%'maxabs';%[-0.01 0.01];% 'maxabs';%[-0.075 0.075]; %'maxabs';%
    cfg.style = 'both';
    cfg.marker = 'off';
    
    % plot
    subplot(3,1,i_cond)
    ft_topoplotER(cfg,TOPO);
    
    
end

annotation('textbox',[0.07 0.98 0.35 0.025],'String',...
    ['TRF lags: ' num2str(plottimes(1)) ' - ' num2str(plottimes(2)) ' s'],'LineStyle','no');

print('-dpdf',[PicDir filesep 'EEGII_TRF_topo_TONE_' num2str(plottimes(1)) ' - ' num2str(plottimes(2))   '.pdf'], '-painters')
%%
figure;
    
    % insert the topodata from each condition into a fieldtrip structure
    clear tmpdat TOPO
    tmpdat = DATA;
    
    cfg = [];
    cfg.latency = [1];
    cfg.trials = 1;
    tmpdat = ft_selectdata(cfg,tmpdat);
    
    cfg =[];
    TOPO = ft_timelockanalysis(cfg,tmpdat); % timelock
    
    TOPO.avg = squeeze(mean(mean(allsbs_BMAT(:,:,plotlag1:plotlag2),3)))'; % replace by real data avg
    
    % ft_timelock that structure in the desired interval
    cfg = [];
    cfg.layout =  'elec1005.lay';
    cfg.colorbar = 'yes';
    cfg.parameter =  'avg';
    cfg.comment = 'no';
    cfg.zlim = 'absmax';%'maxabs';%[-0.01 0.01];% 'maxabs';%[-0.075 0.075]; %'maxabs';%
    cfg.style = 'both';
    cfg.marker = 'off';
    
    % plot
    
    ft_topoplotER(cfg,TOPO);
    


annotation('textbox',[0.07 0.98 0.35 0.025],'String',...
    ['TRF lags: ' num2str(plottimes(1)) ' - ' num2str(plottimes(2)) ' s'],'LineStyle','no');

print('-dpdf',[PicDir filesep 'EEGII_TRF_topo_TONE_avgCond_' num2str(plottimes(1)) ' - ' num2str(plottimes(2))   '.pdf'], '-painters')