
% Script calculates a forward encoding model on time domain EEG DATA
% (see Lalor 2006, 2009);


% Sophie Herbst, Feb. 2016
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%
clear all; clc; close all;
% rng('shuffle');
% make random selection repeatable
rng('default');
rng(1);

%% Directories
% load the path where fieldtrip is located
addpath('/Users/sophie/ownCloud/MATLAB_Toolboxes/fieldtrip-20160620/'); % using the newer FT version!
addpath('/Users/sophie/ownCloud/MATLAB_Toolboxes/shadedErrorBar');
ft_defaults

% load TRF functions
addpath('/Users/sophie/ownCloud/MATLAB_Toolboxes/mTRF_1.3')

rootDir = '/Volumes/SophiesDATA/DisTime_EEGII/';
rawDir = '/Volumes/SophiesDATA/DisTime_EEGII/raw/';
restDir = '/Volumes/SophiesDATA/DisTime_EEGII/RestingState/';

preprocDir = '/Volumes/SophiesDATA/DisTime_EEGII/preproc/';

PicDir = '/Users/sophie/ownCloud/PROJECT_DisTime/DisTime_mTRF/MS_DisTime_TRF/Pics_MS_DisTime_TRF/MatlabOutput/';
resDir =  '/Users/sophie/Documents/DATA/TRF_DisTime_EEGII/';

% colors for the plots
cond_colors = [253,184,99; 178,171,210; 94,60,153];
cond_colors = cond_colors./255;
%% Model Parameters

% run mTFR and prediction or just load the saved data
run_model = 1;

check_plots = 0; % make single subject plots

% lags and lambda for the TFR;
tau = [-200  600];
% timevector for the lags (used for plotting)
tlags = tau(1)/1000:1/200:tau(2)/1000;
nlags = size(tlags,2);

lambda = 4;%4; %0.01;

% epoch time-locked to cross/cue or tone
lock = 'cross';

% latency for ft_selectdata
latency =[0 3.5];

% use only trials that have minimally this foreperiod duration
minFP = 0.65;

% take DATA only from that timepoint (postcue); avoid mixing in cue-resonse
startLAT = .5;

% define one channel for plotting
channame = 'Cz';

%% subjects

sbs = dir([rawDir '*.cnt']);

% restSBS
clear restsbs;
restsbs =  dir([restDir,'*ICAed.mat']);
nrest = numel(restsbs);

% repeat 4 restsbs to have 24
I = randsample(nrest,4);
restsbs(nrest+1) = restsbs(I(1));
restsbs(nrest+2) = restsbs(I(2));
restsbs(nrest+3) = restsbs(I(3));
restsbs(nrest+4) = restsbs(I(4));

%%
colors  = [246 143 39; 215 32 39; 44 130 186 ];%colormap(parula(3));
colors = colors./255;

%% load and plot the hazard Functions:

% load([rootDir filesep 'preproc_notes' filesep  'sHF4Reg_long0.mat']); % subjective (with Shadlen's smoothing factor)
% load([preprocDir filesep  'EEGII_HF4Reg_long0.mat']);
load([preprocDir filesep  'EEGII_HF4Reg_longINT.mat']);

% plot the HFs
figure; hold on;
for i_cond = 1:size(HF_time_matrix_int,2)
    subplot(3,1,i_cond);
    plot(HF_time_matrix_int(i_cond).HF_time(:,1),HF_time_matrix_int(i_cond).HF_time(:,2),...
        'color',cond_colors(i_cond,:),'LineWidth',1);
    title(['condition ' num2str(i_cond)])
    ylabel('hazard');
    ylim([0 1])
end
xlabel('time [s]');

% use only 2 HFs: from conditions 1 & 3
condHF1 = HF_time_matrix_int(1).HF_time(:,2);
condHF3 = HF_time_matrix_int(3).HF_time(:,2);

condHF3 = condHF3-condHF1;
condHF3(find(condHF3 < 0)) = 0;

condHF = [condHF1 condHF3];

[b,a] = butter(6,[25/(200/2)],'low'); % 1.arg = order, 2. = cutoff-freq/(fs/2), 3. low or highpass
condHF = filtfilt(b,a,condHF); % filter twopass

subplot(3,1,1);hold on;
plot(HF_time_matrix_int(1).HF_time(:,1),condHF(:,1),...
    'color',cond_colors(1,:),'LineWidth',1);

subplot(3,1,3);hold on;
plot(HF_time_matrix_int(3).HF_time(:,1),condHF(:,2),...
    'color',cond_colors(i_cond,:),'LineWidth',1);

%% initialize MATRICES sbs x conditions x HF

B_MAT= cell(size(sbs,1),3); % Matrix for the regression weights, sbs x conds
const_MAT = cell(size(sbs,1),3); % Matrix for the regression constant, sbs x conds
origEEG = cell(size(sbs,1),3); % Matrix for original data; does not differ between 2 HFs
STIM_MAT = cell(size(sbs,1),3); % Matrix for Stimuli (HF),sbs x conds

predEEG = cell(size(sbs,1),3,3); % Matrix for predicted EEG data
CORR_MAT = cell(size(sbs,1),3); % Matrix for correlations
CORR_IND = cell(size(sbs,1),3); % Matrix for correlation indices

FP_MAT = cell(size(sbs,1),3); % note the FPs for each trial

% Matrix for trial-averaged reg weights
avgB_MAT = NaN(numel(sbs),3,2,26,nlags);  % sbs x conds x HF x channels x lags

%% start loading DATA

if run_model
    for i_sb = 1 : size(sbs,1)
        
        % load the data DATA for each SB
        % this is the task data, we need it to get the foreperiod,
        % condition, ..
        
        [dum SB] =  fileparts(sbs(i_sb).name);
        disp(['loading subject ' SB '...']);
        load([preprocDir filesep SB 'preproc_complete_' lock 'lock_HP_minphase']);
        
        DATA = DATA_clean; clear DATA_clean;
        
        plotchan_dat = find(strcmp(DATA.label,channame));
        
        %% LP filter the data
        % try to use same filter for stimulus envelope  DATA
        
        cfg = [];
        cfg.lpfilter = 'yes';
        cfg.lpfilttype = 'but'; %'firws';
        
        cfg.lpfilord = 6;
        
        cfg.plotfiltresp = 'no';
        cfg.lpfiltdir = 'twopass'; %'onepass-zerophase';
        % cfg.lpfiltwintype = 'kaiser';
        
        cfg.lpfreq   = 25; %8% % low pass frequency
        cfg.lpfiltdf = 1; % transition band witdth
        
        DATA_filt = ft_preprocessing(cfg,DATA);
        
        if check_plots
            % check effect of filtering
            
            tmptime = DATA.time{1};
            
            cfg = [];
            DATA_avg = ft_timelockanalysis(cfg,DATA);
            DATA_filt_avg = ft_timelockanalysis(cfg,DATA_filt);
            
            tmpdat = DATA_avg.avg;
            tmpdat_filt = DATA_filt_avg.avg;
            
            figure();
            subplot(2,1,1)
            plot(tmptime,tmpdat(plotchan_dat,:)); hold all;
            plot(tmptime,tmpdat_filt(plotchan_dat,:));
            
            legend({'unfiltered', 'filtered'});
            title (['SB ' SB ' channel channame' ])
            
            clear DATA_avg DATA_filt_avg tmpdat tmpdat_filt
        end
        
        DATA = DATA_filt; clear DATA_filt;
        fs = round(DATA.fsample); % round because in some sbs fs = 200.00 and that causes prblms with mTRF
        
        
        % select DATA at the defined latencies
        cfg = [];
        cfg.latency = latency;
        DATA = ft_selectdata(cfg,DATA);
        Tvec = DATA.time{1}; % the data time vector
        
        % for TRF only data from 500ms after cue onset will be used to remove visual onset ERP
        startIX = nearest(Tvec,startLAT); % same for all trials
        
        % endIX is defined in each trial
        
        %% load and filter resting state data
        
        % load resting state data (eo_preproc)
        clear eo_preproc
        [dum rSB] =  fileparts(restsbs(i_sb).name);
        disp(['loading resting state data from subject ' rSB '...']);
        load([restDir filesep rSB]);
        
%         % remove the two A channels
%         cfg = [];
%         channum = [find(strcmp('A1',eo_preproc.label)==1),find(strcmp('A2',eo_preproc.label)==1)];
%         cfg.channel = 1:numel(eo_preproc.label);
%         cfg.channel(channum) = [];
%         eo_preproc = ft_selectdata(cfg,eo_preproc);
        
        % find plotchannel
        plotchan = find(strcmp(eo_preproc.label,channame));
        
        % LP filter the resting state data
        cfg = [];
        cfg.lpfilter = 'yes';
        cfg.lpfilttype = 'but'; %'firws';
        
        cfg.lpfilord = 6;
        
        cfg.plotfiltresp = 'no';
        cfg.lpfiltdir = 'twopass'; %'onepass-zerophase';
        % cfg.lpfiltwintype = 'kaiser';
        
        cfg.lpfreq   = 25; %8% % low pass frequency
        cfg.lpfiltdf = 1; % transition band witdth
        
        eo_preproc_filt = ft_preprocessing(cfg,eo_preproc);
        
        if check_plots
            
            tmptimer = eo_preproc.time{1};
            
            cfg = [];
            DATAr_avg = ft_timelockanalysis(cfg,eo_preproc);
            DATAr_filt_avg = ft_timelockanalysis(cfg,eo_preproc_filt);
            
            tmpdatr = DATAr_avg.avg;
            tmpdatr_filt = DATAr_filt_avg.avg;
            
            subplot(2,1,2)
            plot(tmptimer,tmpdatr(plotchan,:)); hold all;
            plot(tmptimer,tmpdatr_filt(plotchan,:));
            
            legend({'unfiltered', 'filtered'});
            title (['resting state data'])
        end
        
        eo_preproc = eo_preproc_filt; clear eo_preproc_filt;
        
        %% loop over conditions
        for i_cond = 1:3
            disp(['cond ' num2str(i_cond)]);
            
            %% prepare condition DATA
            clear condDATA;
            
            % select all trials from the condition and with FP >= minFP
            cfg = [];
            cfg.trials = find(DATA.trialinfo(:,1) == i_cond & DATA.trialinfo(:,3) >= minFP);
            condDATA = ft_selectdata(cfg,DATA);
            
            
            % initialize subjects's matrix for regression weights: trials x channels x lags
            nchans = size(eo_preproc.trial{1},1);
            
            B_MAT{i_sb,i_cond} = nan(size(condDATA.trial,2),2,nchans,nlags);
            const_MAT{i_sb,i_cond} = nan(size(condDATA.trial,2),2,nchans);
              
            STIM_MAT{i_sb,i_cond} = nan(size(condDATA.trial,2),size(condDATA.trial{1},2),2); % trials x TP x HF
            origEEG{i_sb,i_cond}  = nan(size(condDATA.trial,2),nchans,size(condDATA.trial{1},2)); % trials x chan x TP
          
            
            %% loop over trials
            for i_trial = 1:size(condDATA.trial,2)
                
                % disp(['trial ' num2str(i_trial)]);
                
                trialFP = condDATA.trialinfo(i_trial,3);% FP on each trial
                ixFP = nearest(Tvec,trialFP); % find the closest sampling point
                
                endFP = 3.5;
                ixendFP = nearest(Tvec,endFP);
                
                % trial_Tvec = Tvec(startIX:ixFP); % time vector for the trial DATA
                trial_Tvec = Tvec(startIX:ixendFP); % time vector for the trial DATA
                
                % find a random starting point in the rest data
                % at least 5 secs from end
                % datlength = ixFP - startIX;
                datlength = ixendFP - startIX;
                rand_startIXP = randsample(size(eo_preproc.trial{1},2)-(5*fs),1);
                
                % and cut out the relevant time interval
                trialDAT =  eo_preproc.trial{1}; % rand(size(condDATA.trial{i_trial})); %
                trialDAT = trialDAT(:,rand_startIXP:rand_startIXP+datlength);
                trialDAT(:,(ixFP-startIX)+1:end) = 0; %
                
                % cut the HF to the same lentght
                % trialHF = condHF(startIX:ixFP,:);
                trialHF = condHF(startIX:ixendFP,:);
                
                
                %% zscore both timeseries, make time 1. dimension
                % ztrialHF = zscore(trialHF);
                % normalize HF
                ztrialHF1 = trialHF(:,1) ./ max(trialHF(:,1));
                ztrialHF2 = trialHF(:,2) ./ max(trialHF(:,2));
                ztrialHF = [ztrialHF1 ztrialHF2];
                clear ztrialHF1 ztrialHF2
                
                ztrialDAT = zscore(trialDAT');
                % set the end of the EEG to 0
                ztrialDAT((ixFP-startIX)+1:end,:) = 0; %
                
%                 
%                                 figure;
%                                 plot(trial_Tvec, ztrialHF); hold all;
%                                 plot(trial_Tvec, ztrialDAT(:,plotchan)');
%                                 % plot(trial_Tvec, trialDAT(plotchan,:)');
%                                 legend({'HF', 'EEG data'});
%                                 hline(0,'k');
%                                 title(['Example HF and EEG DATA, 1 Trial']);
%                 
                STIM_MAT{i_sb,i_cond}(i_trial,startIX:ixendFP,:) = ztrialHF;
                origEEG{i_sb,i_cond}(i_trial,:,startIX:ixendFP)  = ztrialDAT';
                
                %% this is the TRF
                [X,T,const] =  mTRFtrain(ztrialHF,ztrialDAT,fs,0,tau(1), tau(2),lambda);
                
                % plot(T,squeeze(X(:,:,plotchan))); % plot single trial mTRF
                
                % write the betas into a matrix
                % write the betas into a matrix
                B_MAT{i_sb,i_cond}(i_trial,:,:,:) = permute(X,[1,3,2]);
                const_MAT{i_sb,i_cond}(i_trial,:,:) = const;
                
                % plot single TRF
                %                                     figure();
                %                                     plot(tlags,squeeze(X(:,:,plotchan)));hold all;
                %                                     hline(0, 'k');
                %                                     title(['TRF SB ' SB ' lambda: ' num2str(lambda) '-- cond' num2str(i_cond) ' -- HF' num2str(i_HF) ' --FP: ' num2str(trialFP)]);
                %                                     xlabel('TRF lag [s]')
                
            end
            
            % avg Bs over trials
            avgB_MAT(i_sb,i_cond,:,:,:) = squeeze(mean(B_MAT{i_sb,i_cond},1));
            
            
        end
        
        
        %% now predict EEG data, 3x3 times:
        % for each condition, crossed with the 3 models
        for i_cond = 1:3
            
            clear condDATA;
            
            % select all trials from the condition and with FP >= minFP
            cfg = [];
            cfg.trials = find(DATA.trialinfo(:,1) == i_cond & DATA.trialinfo(:,3) >= minFP);
            condDATA = ft_selectdata(cfg,DATA);
            
            % loop over trials again, taking out each trial and trying to
            % repredict the EEG from them mean TRF of all other trials
            for i_trial = 1:size(condDATA.trial,2)
                
                % loop over models
                for pred_cond = 1:3
                    
                    if i_trial == 1 % initialize matrix of NaN
                        predEEG{i_sb,i_cond,pred_cond} = nan(size(condDATA.trial,2),size(eo_preproc.trial{1},1),size(condDATA.trial{1},2)); % trials x chan x TP
                    end
                    
                    % Get the model: for condition 1, take models 1, 2 and
                    % 3 and so on = 9 predicted EEG signals
                    % get all weights, dimensions are trials x chans x lags
                    B_leaveout = B_MAT{i_sb,pred_cond};
                    
                    % get mean model constant
                    const_leaveout = const_MAT{i_sb,pred_cond};
                    
                    % if model and data come from the same condition
                    % take out the actual trial
                    if i_cond == pred_cond
                        B_leaveout(i_trial,:,:,:) = [];
                        const_leaveout(i_trial,:,:) = [];
                    end
                    
                    % avg model over trials
                    B_mean_leaveout = squeeze(mean(B_leaveout,1));
                    const_mean_leaveout = squeeze(mean(const_leaveout,1));
                    
                    % prepare the data: from startIX to trialFP
                    trialFP = condDATA.trialinfo(i_trial,3);% FP on each trial
                    endFP = 3.5;% condDATA.trialinfo(i_trial,3);% FP on each trial
                    ixendFP = nearest(Tvec,endFP); % find the closest sampling point
                    
                    % get the original DATA for the trial
                    ztrialDAT =  squeeze(origEEG{i_sb,i_cond}(i_trial,:,startIX:ixendFP))';
                    trial_Tvec = Tvec(startIX:ixendFP); % time vector for the trial DATA
                    
                    % cut the HF to the same interval as the EEG data
                    %  trialHF = condHF(startIX:ixFP,:);
                    trialHF = condHF(startIX:ixendFP,:);
                    
                    % normalize HF
                    ztrialHF1 = trialHF(:,1) ./ max(trialHF(:,1));
                    ztrialHF2 = trialHF(:,2) ./ max(trialHF(:,2));
                    ztrialHF = [ztrialHF1 ztrialHF2];
                    clear ztrialHF1 ztrialHF2
                    
%                                       figure;
%                                       plot(trial_Tvec, ztrialHF(:,1)); hold all;
%                                       plot(trial_Tvec, ztrialHF(:,2)); hold all;
%                                       hline(0,'k');
%                                       plot(trial_Tvec, ztrialDAT(:,plotchan));
%                                       legend({'HF1','HF3', 'EEG data'});
%                     
                    
                    % predict the data of the trial, taking the mean data
                    % of all other trials and the mean model from all other
                    % trials
                    %[pred,rho,pVal,MSE] = mTRFpredict(stim,resp,model,Fs,dir,start,fin,const)
                    clear const M
                    
                    % model: obs x lags x chans
                    M = B_mean_leaveout;
                    s = size(M);
                    
                    % M1 = reshape(M,s(1),s(3),s(2));
                    M = permute(M,[1,3,2]);
                    
                    const = const_mean_leaveout;
                    
                    % here comes the prediction
                    [pE] = mTRFpredict(ztrialHF,ztrialDAT,M,fs,0,tau(1), tau(2),const);
                    
                    % now cut the predicted EEG after the FP
                    ixFP = nearest(Tvec,trialFP); % find the closest sampling point
                    % we already cut the first 500ms, so take that into
                    % account
                    pE_cut = pE(1:((ixFP-startIX)+1),:);
                    
                    % save predicted EEG
                    %predEEG{i_sb,i_cond,pred_cond}(i_trial,:,startIX:ixFP) = pE_cut'; % trials x chan x TP
                    predEEG{i_sb,i_cond,pred_cond}(i_trial,:,startIX:ixendFP) = pE'; % trials x chan x TP
                    
                    
                    if check_plots & i_trial == 100
                        figure;
                        plot(trial_Tvec, ztrialHF,'k'); hold all;
                        plot(trial_Tvec, ztrialDAT(:,plotchan));
                        plot(trial_Tvec, pE(:,plotchan)');
                        legend({'HF1','HF2', 'EEG data', 'predicted EEG data'});
                        hline(0,'k');
                        %                         title(['Example HF and EEG DATA, cond ' num2str(i_cond) ' HF ' num2str(i_HF) ' --FP: ' num2str(trialFP)]);
                        %                         %                     print('-dpdf',[PicDir filesep 'EEGII_TRF_predEEG_SB' num2str(i_sb) ...
                        %                         '_cond_' num2str(i_cond) '_HF_' num2str(i_HF) '_trial_' num2str(i_trial) '.pdf'], '-painters')% PicDir filesep
                    end
                    
                    %% correlate predicted and original EEG
                    
                    for i_chan = 1:size(predEEG{i_sb,i_cond,pred_cond},2)
                        
                        % extract time series of modelled EEG for 1 trial &
                        % channel
                        
                        % modEEG = squeeze(predEEG{i_sb,i_cond,pred_cond}(i_trial,i_chan,:));
                        modEEG = squeeze(predEEG{i_sb,i_cond,pred_cond}(i_trial,i_chan,startIX:ixFP));
                      %  modEEG = modEEG(find(~isnan(modEEG)));
                        
                        % extract time series of real EEG for 1 trial &
                        % channel
                        realEEG = squeeze(origEEG{i_sb,i_cond}(i_trial,i_chan,startIX:ixFP));
                     %   realEEG = realEEG(find(~isnan(realEEG)));
                        
                        % z-score both timeseries
                        modEEGz = zscore(modEEG);
                        realEEGz = zscore(realEEG);
                        
                        % correlate
                        [r,p] = corr(modEEGz,realEEGz);
                        
                        % Fisher z-transform the correlation
                        zr = .5.*log((1+r)./(1-r));
                        
                        % write transformed correlations into matrix
                        CORR_MAT{i_sb,i_cond,pred_cond}(i_trial,i_chan) =   zr;
                    end
                    
                end
            end
            % figure;imagesc(squeeze(predEEG{i_sb,i_cond,pred_cond}(:,1,:)))
        end
        
        %% plot single sb TRFs
        
        figure(i_sb);
        % plot single subject TRF at one channel
        TRF_SB = squeeze(avgB_MAT(i_sb,:,:,plotchan,:));
        for  i_HF = 1:2
            subplot(1,2,i_HF);hold on;
            for i_cond = 1:3
                plot(tlags,squeeze(TRF_SB(i_cond,i_HF,:)),'color',cond_colors(i_cond,:)); hold all;
            end
            
            %ylim([0 ,.01])
            xlabel('TRF lag [s]');
            title(['HF ' num2str(i_HF) ' TRF (lambda = ' num2str(lambda) ')-- ' channame]);
        end
        legend({'cond 1', 'cond 2', 'cond 3'})
    end
    % save([resDir filesep 'RES_EEGII_TRF_pred2HF_lambda' num2str(lambda)],'avgB_MAT')  ;
else
    load ([resDir filesep 'RES_EEGII_TRF_pred2HF_lambda' num2str(lambda)]) ;
end


%% plot avg TRFs
% GFP over lags

my_lines = {'-','--'};

% compute GFP as std over channels
GFP = squeeze(std(avgB_MAT,[],4)); % dim 4 = channels

% avg over sbs and calculate SEM
avgGFP = squeeze(mean(GFP,1));
SEGFP = squeeze(std(GFP,[],1)/sqrt(numel(sbs)));

lags = tlags; % lag vector for plotting

for  i_HF = 1:2 % i_cond = 1:3
    % plot each cond
    figure('Units', 'pixels', ...
        'Position', [100 100 500 375]);
    hold on;
    
    for i_cond = 1:3% i_HF = 1:2
        plot(lags,squeeze(avgGFP(i_cond,i_HF,:)),'color',cond_colors(i_cond,:),...
            'LineStyle',my_lines{i_HF},'LineWidth',1); hold all;
    end
    legend({'cond 1', 'cond 2', 'cond 3'})
    % legend({'monotonous HF', 'peak HF'})
    
    for i_cond = 1:3 % i_HF = 1:2
        shadedErrorBar(lags,squeeze(avgGFP(i_cond,i_HF,:)),squeeze(SEGFP(i_cond,i_HF,:)),{'color',cond_colors(i_cond,:)},.5);
    end
    
    ylim([0 ,.04])
    xlim([-.1 ,.5])
    xlabel('TRF lag [s]');
    
    xlabel('TRF lag [s]');
    
    
    hl = hline(0,'k')
    set(hl,'color',[.3 .3 .3])
    vl = vline(0,'k')
    set(vl,'color',[.3 .3 .3])
    
    % title(['cond ' num2str(i_cond) ' TRF (lambda = ' num2str(lambda) ') : GFP']);
    title(['HF ' num2str(i_HF) ' TRF (lambda = ' num2str(lambda) ') : GFP']);
    
    
    % aesthetics
    set( gca                       , ...
        'FontName'   , 'Helvetica' ,...
        'FontSize'   , 10 );
    
    set(gca, ...
        'Box'         , 'off'     , ...
        'TickDir'     , 'out'     , ...
        'TickLength'  , [.02 .02] , ...
        'XMinorTick'  , 'off'      , ...
        'YMinorTick'  , 'off'      , ...
        'YGrid'       , 'off'      , ...
        'XGrid'       , 'off'      , ...
        'XColor'      , [0 0 0], ...
        'YColor'      , [0 0 0 ], ...
        'YTick'       , 0: .01 : .04, ...
        'LineWidth'   , .5         );
    
    % print( '-dpdf',[PicDir filesep 'EEGII_TRF_GFP_2HF_cond' num2str(i_cond) '.pdf'], '-painters')% PicDir filesep
    print( '-dpdf',[PicDir filesep 'EEGII_TRF_GFP_2HF_restDATA_HF' num2str(i_HF) '.pdf'], '-painters')% PicDir filesep
    
end


%% plot avg TRFs at one channel

% plot each cond


thischan = 'Fz';
plotchan = find(strcmp(eo_preproc.label,thischan));

% compute AVG over channels
chan_TRF = squeeze(avgB_MAT(:,:,:,plotchan,:)); % dim 4 = channels

% avg over sbs and calculate SEM
avgTRF = squeeze(mean(chan_TRF,1));
seTRF = squeeze(std(chan_TRF,[],1)/sqrt(numel(sbs)));

lags = tlags; % lag vector for plotting

for i_HF = 1:2 %i_cond = 1:3
    
    figure('Units', 'pixels', ...
        'Position', [100 100 500 375]);
    hold on;
    
    % plot each cond
    
    for i_cond = 1:3% i_HF = 1:2
        
        plot(lags,squeeze(avgTRF(i_cond,i_HF,:)),'color',cond_colors(i_cond,:),...
            'LineStyle',my_lines{i_HF},'LineWidth',1); hold all;
    end
    legend({'cond 1', 'cond 2', 'cond 3'})
    % legend({'monotonous HF', 'peak HF'})
    
    for i_cond = 1:3 % i_HF = 1:2
        shadedErrorBar(lags,squeeze(avgTRF(i_cond,i_HF,:)),squeeze(seTRF(i_cond,i_HF,:)),{'color',cond_colors(i_cond,:)},.5);
    end
    
    ylim([-0.06 0.05])
    xlim([-0.1 0.5])
    xlabel('TRF lag [s]');
    
    
    xlabel('TRF lag [s]');
    hl = hline(0,'k')
    set(hl,'color',[.3 .3 .3])
    vl = vline(0,'k')
    set(vl,'color',[.3 .3 .3])
    
    % title(['TRF cond ' num2str(i_cond) '(lambda=' num2str(lambda) ') ' thischan]);
    title(['TRF HF ' num2str(i_HF) '(lambda=' num2str(lambda) ') ' thischan]);
    
    % aesthetics
    set( gca                       , ...
        'FontName'   , 'Helvetica' ,...
        'FontSize'   , 8 );
    
    set(gca, ...
        'Box'         , 'off'     , ...
        'TickDir'     , 'out'     , ...
        'TickLength'  , [.02 .02] , ...
        'XMinorTick'  , 'off'      , ...
        'YMinorTick'  , 'off'      , ...
        'YGrid'       , 'off'      , ...
        'XGrid'       , 'off'      , ...
        'XColor'      , [0 0 0], ...
        'YColor'      , [0 0 0], ...
        'YTick'       , -.1: .05 : .05, ...
        'LineWidth'   , .5         );
    
    
    %  print('-dpdf',[PicDir filesep 'EEGII_TRF_2HFs' thischan '_cond' num2str(i_cond) '.pdf'], '-painters')% PicDir filesep
    print('-dpdf',[PicDir filesep 'EEGII_TRF_2HFs_restDATA' thischan '_HF' num2str(i_HF) '.pdf'], '-painters')% PicDir filesep
    
end

%% plot topos of regression weights

% avg weights over sbs
allsbs_BMAT = squeeze(mean(avgB_MAT,1)); % first dim is now cond

% lags at which to plot
plottimes = [-0.1 0.01];%[0.2 0.3];%[-0.01 0.01];% [0.2 0.3];% plottimes = [-0.01 0.01]; % %[-0.01 001]; %[0.15 0.25];%[0 0]
plotlag1 =  nearest(lags,plottimes(1));
plotlag2 =  nearest(lags,plottimes(2));

for i_HF = 2%:2
    figure;
    for i_cond = 1:3
        
        % insert the topodata from each condition into a fieldtrip structure
        clear tmpdat TOPO
        tmpdat = eo_preproc;
        
        cfg = [];
        cfg.latency = [1];
        cfg.trials = 1;
        tmpdat = ft_selectdata(cfg,tmpdat);
        
        cfg =[];
        TOPO = ft_timelockanalysis(cfg,tmpdat); % timelock
        
        TOPO.avg = squeeze(mean(allsbs_BMAT(i_cond,i_HF,:,plotlag1:plotlag2),4)); % replace by real data avg
        
        % ft_timelock that structure in the desired interval
        cfg = [];
        cfg.layout =  'rest_EEG_electrodes.lay';
        cfg.colorbar = 'yes';
        cfg.parameter =  'avg';
        cfg.comment = 'no';
        cfg.zlim =  [-0.005 0.005];%[-0.01 0.01];
        cfg.style = 'straight';
        cfg.marker = 'off';
        
        
        % plot
        subplot(3,1,i_cond)
        ft_topoplotER(cfg,TOPO);
        
        
    end
    
    annotation('textbox',[0.07 0.98 0.35 0.025],'String',...
        ['HF ' num2str(i_HF) ' TRF lags: ' num2str(plottimes(1)) ' - ' num2str(plottimes(2)) ' s'],'LineStyle','no');
    
    print('-dpdf',[PicDir filesep 'EEGII_TRF_topo_restDATA_HF' num2str(i_HF) '_' num2str(plottimes(1)) ' - ' num2str(plottimes(2))   '.pdf'], '-painters')% PicDir filesep
    
end

%% compute correlation index

% datacond is the condition where the original EEG comes from
% predcond (the 3rd dimension of CORR_MAT is the condition from which the
% model was estimated

for i_sb = 1:size(CORR_MAT,1)
    for i_datacond = 1:size(CORR_MAT,2)
        for i_trial =   1: size(CORR_MAT{i_sb,i_datacond,1},1)
            
            corr1 = CORR_MAT{i_sb,i_datacond,1}(i_trial,:);
            corr3 = CORR_MAT{i_sb,i_datacond,3}(i_trial,:);
           
            ind_tmp = robust_index(corr1,corr3);
            
            CORR_IND{i_sb,i_datacond}(i_trial,:) = ind_tmp;
            
        end
        avg_CORR_IND(i_sb,i_datacond,:) = squeeze(mean(CORR_IND{i_sb,i_datacond},1));
    end
end

% add boxplot per condition
figure();hold on;
thischan = 'Fz';

plotchan = find(strcmp(eo_preproc.label,thischan));
plot([1 2 3],mean(avg_CORR_IND(:,:,plotchan),1),...
        'Color',[0 0 0],'LineWidth',1);
    
for i_sb = 1:size(avg_CORR_IND,1)
    plot([1 2 3],avg_CORR_IND(i_sb,:,plotchan),'LineWidth',0.5,'Color',[0.9 0.9 0.9]);
end

for i_cond = 1:3
    plot(i_cond,avg_CORR_IND(:,i_cond,plotchan),'Marker','*','Color',[0.8 0.8 0.8]);
    plot(i_cond,mean(avg_CORR_IND(:,i_cond,plotchan),1),...
        'Marker','*','Color',cond_colors(i_cond,:),'LineWidth',2);
end

ylim([-.04 .04]);
xlim([0.75 3.25]);
hline(0,'k');

xlabel(['data cond']);
ylabel(['robust index'])
title(['correlation index (model 1 - 3) ' thischan])

print('-dpdf',[PicDir filesep 'EEGII_TRF_2HFsREST_CorrInd_' thischan '.pdf'], '-painters')% PicDir filesep

%% plot topos of CorrInd
figure;
for i_cond = 1:3
    
    % insert the topodata from each condition into a fieldtrip structure
    clear tmpdat TOPO
    tmpdat = eo_preproc;
    
    cfg = [];
    cfg.latency = [1];
    cfg.trials = 1;
    tmpdat = ft_selectdata(cfg,tmpdat);
    
    cfg =[];
    TOPO = ft_timelockanalysis(cfg,tmpdat); % timelock
    
    TOPO.avg = squeeze(mean(avg_CORR_IND(:,i_cond,:)));
    
    % ft_timelock that structure in the desired interval
    cfg = [];
    cfg.layout =  'rest_EEG_electrodes.lay';
    cfg.colorbar = 'yes';
    cfg.parameter =  'avg';
    cfg.comment = 'no';
    cfg.zlim =  [-0.005 0.005]; %'maxabs';%
    cfg.style = 'straight';
    cfg.marker = 'off';
    
    
    % plot
    subplot(3,1,i_cond)
    ft_topoplotER(cfg,TOPO);
    
    
end

%     annotation('textbox',[0.07 0.98 0.35 0.025],'String',...
%         ['HF ' num2str(i_HF) ' TRF lags: ' num2str(plottimes(1)) ' - ' num2str(plottimes(2)) ' s'],'LineStyle','no');
%
print('-dpdf',[PicDir filesep 'EEGII_TRF_restDATA_2HFs_CorrInd_Topos.pdf'], '-painters')% PicDir filesep

%% compute running average of correlations over trials

colors = parula(4);
colors(3,:) = [];

maxtrials = 200; % maximal number of trials per cond
corrmat_trials_smooth = nan(numel(sbs),3,3,maxtrials,nchans);
figure();

% make matrix in which trials are concatenated
for i_sb = 1:numel(sbs)
    for i_cond = 1:3
        for pred_cond = 1:3
            ntrials = size(CORR_MAT{i_sb,i_cond,pred_cond},1);
            
            mean_CORR_MAT(i_sb,i_cond,pred_cond,:) = squeeze(nanmean(CORR_MAT{i_sb,i_cond,pred_cond},1));
            
            tmpdat = CORR_MAT{i_sb,i_cond,pred_cond};
            
            %             figure(i_sb + 100);
            %             subplot(1,3,i_cond);hold on;
            %             plot(1:size(tmpdat,1),tmpdat(:,plotchan),'color',colors(i_HF,:));
            %
            
            for i_trial = 8:size(CORR_MAT{i_sb,i_cond,pred_cond},1)-7
                % running avg over 15 trials
                tmpdat_smooth =  squeeze(nanmean(tmpdat(i_trial-7:i_trial+7,:),1));
                corrmat_trials_smooth(i_sb,i_cond,pred_cond,i_trial,:) = tmpdat_smooth;
            end
            subplot(1,3,i_cond);hold on;
            plotdat = squeeze(corrmat_trials_smooth(i_sb,i_cond,pred_cond,:,plotchan));
            plot(1:size(plotdat,1),plotdat,'color',colors(pred_cond,:),'LineWidth',1.5);
            ylim([-1 1])
            
            clear tmpdat;clear tmpdat_smooth;
        end
        
        
    end
end

% avg and sd
mean_corrmat_trials_smooth = squeeze(nanmean(corrmat_trials_smooth,1));
sd_corrmat_trials_smooth = squeeze(nanstd(corrmat_trials_smooth,1))./sqrt(24);


%% plot CORR for single subjects

thischan = 'Fz';
plotchan = find(strcmp(eo_preproc.label,thischan));

figure(101+1);clf;figure(101+2);clf;figure(101+3);clf;
for i_sb = 1:numel(sbs)
    
    for i_cond = 1:3
        figure(101+i_cond); %subplot(6,4,i_sb)
        plot(squeeze(corrmat_trials_smooth(i_sb,i_cond,1,:,plotchan)),'color',cond_colors(1,:),'LineWidth',2) ; hold all;
        plot(squeeze(corrmat_trials_smooth(i_sb,i_cond,2,:,plotchan)),'color',cond_colors(2,:),'LineWidth',2) ; hold all;
        plot(squeeze(corrmat_trials_smooth(i_sb,i_cond,3,:,plotchan)),'color',cond_colors(3,:),'LineWidth',2) ; hold all;
        
        if i_sb == 1
            title(['cond' num2str(i_cond)])
        end
        ylim([-0.5 .5]);
        hline(0,'k');
    end
    
    % vline(condborders_ss(1),'k');
    % vline(condborders_ss(2),'k');
    % vline(condborders_ss(3),'k');
    if i_sb == 1
        legend({'predcond 1','predcond 2','predcond 3'});
    end
    
end
% % print -depsc mTRF_500_CORR_runavg.eps -painters

%% plot Correlations

thischan = 'Fz';
plotchan = find(strcmp(eo_preproc.label,thischan));

figure(201+1);clf;
for i_cond =  1:3
    subplot(3,1,i_cond)
    
    cond_ix = find(~isnan(mean_corrmat_trials_smooth(i_cond,1,:,plotchan)));
    
    plot(cond_ix,squeeze(mean_corrmat_trials_smooth(i_cond,1,cond_ix,plotchan)),'color',cond_colors(1,:),'LineWidth',2) ; hold all;
    plot(cond_ix,squeeze(mean_corrmat_trials_smooth(i_cond,2,cond_ix,plotchan)),'color',cond_colors(2,:),'LineWidth',2) ; hold all;
    plot(cond_ix,squeeze(mean_corrmat_trials_smooth(i_cond,3,cond_ix,plotchan)),'color',cond_colors(3,:),'LineWidth',2) ; hold all;
    
    %     shadedErrorBar(cond_ix,squeeze(mean_corrmat_trials_smooth(i_cond,1,cond_ix,plotchan))',...
    %         squeeze(sd_corrmat_trials_smooth(i_cond,1,cond_ix,plotchan))',{'color',colors(1,:)},.5);
    %
    %     shadedErrorBar(cond_ix,squeeze(mean_corrmat_trials_smooth(i_cond,2,cond_ix,plotchan)),...
    %         squeeze(sd_corrmat_trials_smooth(i_cond,2,cond_ix,plotchan)),{'color',colors(2,:)},.5);
    %
    ylim([-0.1 .1]);
    hline(0,'k');
    legend({'predcond1','predcond2','predcond3'});
    
    title(['correlations per HF channel:  Cond ' num2str(i_cond) ' ' thischan ' lambda: ' num2str(lambda)]);
end

print('-dpdf',[PicDir 'EEGII_TRF_restDATA_CORR' thischan  '.pdf'], '-painters')% PicDir filesep

%% add boxplot per condition

thischan = 'Fz';
plotchan = find(strcmp(eo_preproc.label,thischan));

figure();
for i_sb = 1:24
    for i_datacond = 1:3
        for i_modcond = 1:3
        mean_corr(i_sb,i_datacond,i_modcond) = squeeze(mean(CORR_MAT{i_sb,i_datacond,i_modcond}(:,plotchan),1));
        end
    end
end

for i_datacond = 1:3
    subplot(1,3,i_datacond)
    boxplot(squeeze(mean_corr(:,i_datacond,:)),'plotstyle','compact',...
        'symbol','k.','positions',[1 2 3],'colors',cond_colors,...
        'medianstyle','line');
    
    hline(0,'k');
    ylim([-.1  .3])
    
    grid on;
    title(['data cond: ' num2str(i_datacond)])
end
% make median lines black and big
set(findobj(gcf,'Tag','Median'),'LineWidth',2);

% make outlier dots gray and big
set(findobj(gcf,'Tag','Outliers'),'MarkerEdgeColor',[0.4 0.4 0.4],'MarkerSize',5);

print('-dpdf',[PicDir filesep 'EEGII_TRF_REST_Corr_BOX_' thischan '.pdf'], '-painters')% PicDir filesep

%% plot predicted EEG

% choose one channel
thischan = 'Fz';
plotchan = find(strcmp(eo_preproc.label,thischan));

clear condEEG condEEG_all

% loop over sbs and conds
for i_sb = 1:numel(sbs)
    for i_cond = 1:3
        for i_predcond = 1:3
            % extract data, nanmean over trials (nans exist because of unequal trial durations)
            tmp = squeeze(nanmean(predEEG{i_sb,i_cond,i_predcond},1));
%             tmp = tmp(plotchan,:);
%             tmp = tmp(find(~isnan(tmp)));

            % write in a matrix, using only the above defined channel
            condEEG(i_sb,i_cond,i_predcond,:) = tmp(plotchan,:);
        end
    end
end

% avg over participants
condEEG_all = squeeze(mean(condEEG,1));
SE_condEEG_all = squeeze(std(condEEG,1))./sqrt(numel(sbs));

%% plot pred EEG
figure;

for i_cond = 1%:3 
    % subplot(1,3,i_cond);hold on;
        for i_predcond = 1:3
        plot(Tvec,squeeze(condEEG_all(i_cond,i_predcond,:)),...
            'color',cond_colors(i_predcond,:),...
        'LineWidth',1.5);   hold on;
        end
        hline(0,'k');
        ylim([-.25 .25]);
        title(['datacond ' num2str(i_cond)]);
end

  for i_predcond = 1:3 % i_HF = 1:2
        shadedErrorBar(Tvec,squeeze(condEEG_all(i_cond,i_predcond,:)),squeeze(SE_condEEG_all(i_cond,i_predcond,:)),...
            {'color',cond_colors(i_predcond,:)},.5);
  end

% aesthetics
    set( gca                       , ...
        'FontName'   , 'Helvetica' ,...
        'FontSize'   , 8 );
    
    set(gca, ...
        'Box'         , 'off'     , ...
        'TickDir'     , 'out'     , ...
        'TickLength'  , [.02 .02] , ...
        'XMinorTick'  , 'off'      , ...
        'YMinorTick'  , 'off'      , ...
        'YGrid'       , 'off'      , ...
        'XGrid'       , 'off'      , ...
        'XColor'      , [0 0 0], ...
        'YColor'      , [0 0 0], ...
        'YTick'       , -.25: .05 : .25, ...
        'LineWidth'   , 0.5         );

    title(['predicted EEG ' thischan])
print('-dpdf',[PicDir filesep 'EEGII_TRF_REST_predEEG_' thischan '.pdf'], '-painters')% PicDir filesep
